@@ -1,6 +1,6 @@
 package YATT::Lite::PSGIEnv; sub Env () {__PACKAGE__}
 use strict;
-use warnings qw(FATAL all NONFATAL misc);
+use warnings FATAL => qw(all);
 use Carp;
 
 my @PSGI_FIELDS;
@@ -17,8 +17,6 @@ BEGIN {
 	  PATH_INFO
 	  PATH_TRANSLATED
 	  REDIRECT_STATUS
-          REDIRECT_HANDLER
-          REDIRECT_URL
 	  REQUEST_URI
 	  DOCUMENT_URI
 
@@ -60,8 +58,6 @@ BEGIN {
 	  psgix.session
 	  psgix.session.options
 	  psgix.logger
-
-          yatt.script_name
        /;
   our %FIELDS = map {$_ => ''} @PSGI_FIELDS;
 }
