@@ -1,6 +1,6 @@
 package YATT::Lite::Util::AsBase;
 use strict;
-use warnings qw(FATAL all NONFATAL misc);
+use warnings FATAL => qw/all/;
 use Carp;
 
 require Exporter;
@@ -9,13 +9,9 @@ our @EXPORT_OK = @EXPORT;
 
 use YATT::Lite::Util qw/ckeval globref/;
 
-use constant DEBUG_IMPORT => $ENV{DEBUG_YATT_IMPORT} // 0;
-
-require mro;
 require YATT::Lite::MFields;
 
 sub import {
-  Carp::carp(scalar caller, " calls $_[0]->import()") if DEBUG_IMPORT;
   parse_args(\@_, scalar caller);
   goto &Exporter::import;
 }
@@ -49,9 +45,8 @@ sub _import_as_base {
     unless ($isa = *{$sym}{ARRAY}) {
       *$sym = $isa = [];
     }
-    mro::set_mro($callpack, 'c3');
     unless (grep {$_ eq $myPack} @$isa) {
-      unshift @$isa, $myPack;
+      push @$isa, $myPack;
     }
   }
 
